{{
'-------------------------------------------------------------------------------
'
' Loader.inc - This file is included by all target files to perform the logic 
'              required to include the correct Loader, based on the following
'              symbols:
'
'              TINY    - LMM memory model
'              SDCARD  - use SDCARD Loader
'              EEPROM  - use EEPROM Loader
'              SMALL   - XMM SMALL memory model
'              LARGE   - XMM LARGE memory model
'
'            
' This file is included by the following target files:
'
'   lmm_default.spin
'   emm_default.spin
'   smm_default.spin
'   xmm_default.spin
'   lmm_blackcat.spin
'   emm_blackcat.spin
'   smm_blackcat.spin
'   xmm_blackcat.spin
'
' Version 2.8 - initial version by Ross Higson
'
}}
#ifdef TINY
#ifdef SDCARD
  ' SDCARD two-phase loader for LMM programs
  Loader   : "Catalina_SMM_SDCARD_Loader"
#elseifdef EEPROM
  ' EEPROM two-phase loader for LMM programs
  Loader   : "Catalina_EMM_EEPROM_Loader"
#else
' no loader is required for this program (one-phase loader)
#endif
#elseifdef SMALL
#ifdef EEPROM
  ' EEPROM two-phase loader for XMM programs
  Loader   : "Catalina_XMM_EEPROM_Loader"
#else
  ' Hub Loader is used for both SCARD and Serial Load for XMM programs  
  Loader   : "Catalina_HUB_XMM_Loader"
#endif
#elseifdef LARGE
#ifdef EEPROM
  ' EEPROM two-phase loader for XMM programs
  Loader   : "Catalina_XMM_EEPROM_Loader"
#else  
  ' Hub Loader is used for both SCARD and Serial Load for XMM programs  
  Loader   : "Catalina_HUB_XMM_Loader"
#endif
#else
  ERROR : Cannot determine loader to use
#endif
